------------------------------------------------------
-- GFWZones.lua
-- Utilities for working with geographic data 
------------------------------------------------------

GFWZONES_THIS_VERSION = 4;

------------------------------------------------------

if (GFWZones == nil) then
	GFWZones = {};
end

function GFWZones_temp_NormalizeZoneName(aZone)

	aZone = string.gsub(aZone, "^The (.*)$", "%1");
	aZone = string.gsub(aZone, "^City of (.*)$", "%1");
	
	return aZone;
end

function GFWZones_temp_ConnectionsForZone(aZone)

	local G = GFWZones;
	aZone = G.NormalizeZoneName(aZone);
	
	local zoneConnections = { };
	local myFaction = (GetFactionInfo(1));
	
	if not (G.AdjacentZones[aZone] or G.FlightZones[myFaction][aZone]) then return nil; end
	
	-- find zones one step away (adjacent to this zone or one flight/boat/zeppelin away)
	zoneConnections[1] = GFWTable.Merge(G.AdjacentZones[aZone], G.FlightZones[myFaction][aZone]);
	zoneConnections[1] = GFWTable.Subtract(zoneConnections[1], {aZone});
	
	-- then iterate to find zones more than one step away
	numSteps = 2;
	repeat	
		zoneConnections[numSteps] = { };
		for i=1, table.getn(zoneConnections[numSteps-1]) do
			zoneConnections[numSteps] = GFWTable.Merge(zoneConnections[numSteps], G.FlightZones[myFaction][zoneConnections[numSteps-1][i]]);
			zoneConnections[numSteps] = GFWTable.Merge(zoneConnections[numSteps], G.AdjacentZones[zoneConnections[numSteps-1][i]]);
		end
		for i=numSteps-1, 1, -1 do
			zoneConnections[numSteps] = GFWTable.Subtract(zoneConnections[numSteps], zoneConnections[i]);
		end
		zoneConnections[numSteps] = GFWTable.Subtract(zoneConnections[numSteps], {aZone});
		numSteps = numSteps + 1;
	until (table.getn(zoneConnections[numSteps-1]) == 0 or numSteps == 10); -- don't want to go forever, seems like a reasonable cutoff

	return zoneConnections;

end


------------------------------------------------------
-- Zone Connection Data
------------------------------------------------------

local tempAdjacentZones = {
	["Alterac Mountains"] = {"Silverpine Forest", "Hillsbrad Foothills", "Alterac Valley"},
	["Arathi Highlands"] = {"Wetlands", "Hillsbrad Foothills"},
	["Ashenvale"] = {"Barrens", "Azshara", "Darkshore", "Felwood", "Stonetalon Mountains", "Blackfathom Deeps", "Warsong Gulch"},
	["Azshara"] = {"Ashenvale"},
	["Badlands"] = {"Loch Modan", "Searing Gorge", "Uldaman"},
	["Blackrock Mountain"] = {"Searing Gorge", "Burning Steppes", "Blackrock Depths", "Blackrock Spire", "Molten Core"},
	["Blasted Lands"] = {"Swamp of Sorrows"},
	["Burning Steppes"] = {"Blackrock Mountain", "Redridge Mountains"},
	["Ironforge"] = {"Dun Morogh"},
	["Darkshore"] = {"Ashenvale", "Teldrassil", "Wetlands"}, -- boat
	["Darnassus"] = {"Teldrassil"},
	["Deadwind Pass"] = {"Duskwood", "Swamp of Sorrows"},
	["Desolace"] = {"Stonetalon Mountains", "Feralas", "Desolace", "Maraudon"},
	["Dun Morogh"] = {"Ironforge", "Loch Modan", "Gnomeregan"},
	["Durotar"] = {"Orgrimmar", "Barrens", "Stranglethorn Vale", "Tirisfal Glades"}, -- zeppelin
	["Duskwood"] = {"Elwynn Forest", "Westfall", "Deadwind Pass", "Stranglethorn Vale"},
	["Dustwallow Marsh"] = {"Barrens", "Wetlands"}, -- boat
	["Eastern Plaguelands"] = {"Western Plaguelands", "Stratholme"},
	["Elwynn Forest"] = {"Stormwind City", "Westfall", "Duskwood", "Redridge Mountains"},
	["Felwood"] = {"Ashenvale", "Moonglade", "Winterspring"},
	["Feralas"] = {"Desolace", "Thousand Needles", "Dire Maul"},
	["Hillsbrad Foothills"] = {"Silverpine Forest", "Alterac Mountains", "Arathi Highlands", "Hinterlands"},
	["Loch Modan"] = {"Dun Morogh", "Wetlands", "Badlands"},
	["Moonglade"] = {"Felwood", "Winterspring"},
	["Mulgore"] = {"Thunder Bluff", "Barrens"},
	["Orgrimmar"] = {"Barrens", "Durotar", "Ragefire Chasm"},
	["Redridge Mountains"] = {"Elwynn Forest", "Duskwood", "Burning Steppes"},
	["Searing Gorge"] = {"Badlands", "Blackrock Mountain"},
	["Silithus"] = {"Un'goro Crater", "Ahn'Qiraj"},
	["Silverpine Forest"] = {"Tirisfal Glades", "Hillsbrad Foothills", "Alterac Mountains", "Shadowfang Keep"},
	["Stonetalon Mountains"] = {"Ashenvale", "Barrens", "Desolace"},
	["Stormwind City"] = {"Elwynn Forest", "Stockade"},
	["Stranglethorn Vale"] = {"Duskwood", "Barrens", "Tirisfal Glades", "Durotar", "Zul'Gurub"}, -- boat, zeppelin
	["Swamp of Sorrows"] = {"Deadwind Pass", "Blasted Lands", "Temple of Atal'Hakkar"},
	["Tanaris"] = {"Thousand Needles", "Un'Goro Crater", "Caverns of Time", "Zul'Farrak"},
	["Teldrassil"] = {"Darnassus", "Darkshore"}, -- boat
	["Barrens"] = {"Durotar", "Mulgore", "Ashenvale", "Stonetalon Mountains", "Dustwallow Marsh", "Thousand Needles", "Stranglethorn Vale", "Wailing Caverns", "Razorfen Kraul", "Razorfen Downs", "Warsong Gulch"}, -- boat
	["Hinterlands"] = {"Hillsbrad Foothills", "Western Plaguelands"},
	["Undercity"] = {"Tirisfal Glades"},
	["Thousand Needles"] = {"Barrens", "Feralas", "Tanaris"},
	["Thunder Bluff"] = {"Mulgore"},
	["Tirisfal Glades"] = {"Silverpine Forest", "Undercity", "Western Plaguelands", "Stranglethorn Vale", "Durotar", "Scarlet Monastery"}, -- zeppelin
	["Un'Goro Crater"] = {"Tanaris", "Silithus"},
	["Western Plaguelands"] = {"Tirisfal Glades", "Alterac Mountains", "Hinterlands", "Eastern Plaguelands", "Scholomance"},
	["Westfall"] = {"Elwynn Forest", "Duskwood", "Deadmines"},
	["Wetlands"] = {"Arathi Highlands", "Loch Modan", "Dustwallow Marsh"}, -- boat
	["Winterspring"] = {"Moonglade", "Felwood"},

-- Instances / Battlegrounds / etc.
	["Ahn'Qiraj"] = {"Silithus"},
	["Arathi Basin"] = {"Arathi Highlands"},
	["Alterac Valley"] = {"Alterac Mountains"},
	["Blackfathom Deeps"] = {"Ashenvale"},
	["Blackrock Depths"] = {"Blackrock Mountain", "Molten Core"},
	["Blackrock Spire"] = {"Blackrock Mountain"},
	["Caverns of Time"] = {"Tanaris"},
	["Champions' Hall"] = {"Stormwind City"},
	["Dire Maul"] = {"Feralas"},
	["Gnomeregan"] = {"Dun Morogh"},
	["Hall of Legends"] = {"Orgrimmar"},
	["Maraudon"] = {"Desolace"},
	["Onyxia's Lair"] = {"Dustwallow Marsh"},
	["Ragefire Chasm"] = {"Orgrimmar"},
	["Razorfen Downs"] = {"Barrens", "Thousand Needles"},
	["Razorfen Kraul"] = {"Barrens"},
	["Scarlet Monastery"] = {"Tirisfal Glades"},
	["Scholomance"] = {"Western Plaguelands"},
	["Shadowfang Keep"] = {"Silverpine Forest"},
	["Stratholme"] = {"Eastern Plaguelands"},
	["Deadmines"] = {"Westfall"},
	["Molten Core"] = {"Blackrock Depths", "Blackrock Mountain"},
	["Stockade"] = {"Stormwind City"},
	["Temple of Atal'Hakkar"] = {"Swamp of Sorrows"},
	["Wailing Caverns"] = {"Barrens"},
	["Uldaman"] = {"Badlands"},
	["Warsong Gulch"] = {"Barrens", "Ashenvale"},
	["Zul'Farrak"] = {"Tanaris"},
	["Zul'Gurub"] = {"Stranglethorn Vale"},
};

local tempFlightZones = {
	[FACTION_ALLIANCE] = {
		-- Deeprun Tram is in here even though it's not a "flight" per se because Horde can't easily travel through it. 
		["Arathi Highlands"] = {"Hillsbrad Foothills", "Ironforge", "Hinterlands", "Loch Modan", "Wetlands", "Arathi Basin"},
		["Ashenvale"] = {"Darkshore"},
		["Azshara"] = {"Felwood", "Darkshore"},
		["Blasted Lands"] = {"Duskwood", "Burning Steppes", "Stormwind City"},
		["Burning Steppes"] = {"Blasted Lands", "Searing Gorge"},
		["Ironforge"] = {"Arathi Highlands", "Wetlands", "Hillsbrad Foothills", "Stormwind City", "Loch Modan", "Hinterlands", "Searing Gorge", "Deeprun Tram"},
		["Darkshore"] = {"Teldrassil", "Ashenvale", "Moonglade", "Azshara", "Felwood", "Dustwallow Marsh", "Desolace", "Feralas"},
		["Deeprun Tram"] = {"Ironforge", "Stormwind City"},
		["Desolace"] = {"Darkshore", "Dustwallow Marsh", "Feralas"},
		["Duskwood"] = {"Blasted Lands", "Redridge Mountains", "Westfall", "Stormwind City", "Stranglethorn Vale"},
		["Dustwallow Marsh"] = {"Desolace", "Tanaris", "Darkshore", "Feralas"},
		["Eastern Plaguelands"] = {"Hinterlands"},
		["Felwood"] = {"Winterspring", "Azshara", "Darkshore"},
		["Feralas"] = {"Darkshore", "Desolace", "Tanaris", "Dustwallow Marsh"},
		["Hillsbrad Foothills"] = {"Ironforge", "Wetlands", "Western Plaguelands", "Hinterlands", "Arathi Highlands"},
		["Loch Modan"] = {"Wetlands", "Ironforge", "Arathi Highlands"},
		["Moonglade"] = {"Darkshore", "Winterspring"},
		["Redridge Mountains"] = {"Duskwood", "Westfall", "Stormwind City"},
		["Searing Gorge"] = {"Ironforge", "Burning Steppes"},
		["Silithus"] = {"Tanaris"},
		["Stonetalon Mountains"] = {"Darkshore"},
		["Stormwind City"] = {"Stranglethorn Vale", "Westfall", "Ironforge", "Duskwood", "Redridge Mountains", "Blasted Lands", "Deeprun Tram"},
		["Stranglethorn Vale"] = {"Duskwood", "Westfall", "Stormwind City"},
		["Tanaris"] = {"Dustwallow Marsh", "Silithus", "Feralas"},
		["Teldrassil"] = {"Darkshore"},
		["Hinterlands"] = {"Hillsbrad Foothills", "Ironforge", "Eastern Plaguelands", "Arathi Highlands"},
		["Western Plaguelands"] = {"Hillsbrad Foothills"},
		["Westfall"] = {"Stranglethorn Vale", "Duskwood", "Redridge Mountains", "Stormwind City"},
		["Wetlands"] = {"Loch Modan", "Hillsbrad Foothills", "Arathi Highlands", "Ironforge"},
		["Winterspring"] = {"Moonglade", "Felwood"},
	},
	[FACTION_HORDE] = {
		["Arathi Highlands"] = {"Hillsbrad Foothills", "Undercity", "Badlands"},
		["Ashenvale"] = {"Orgrimmar", "Barrens"},
		["Azshara"] = {"Orgrimmar", "Barrens", "Felwood"},
		["Badlands"] = {"Stranglethorn Vale", "Arathi Highlands", "Swamp of Sorrows", "Undercity", "Searing Gorge"},
		["Burning Steppes"] = {"Searing Gorge"},
		["Desolace"] = {"Stonetalon Mountains", "Thunder Bluff", "Feralas"},
		["Dustwallow Marsh"] = {"Orgrimmar", "Barrens", "Tanaris", "Mulgore"},
		["Eastern Plaguelands"] = {"Undercity"},
		["Felwood"] = {"Moonglade", "Orgrimmar", "Winterspring"},
		["Feralas"] = {"Barrens", "Thunder Bluff", "Tanaris", "Desolace"},
		["Hillsbrad Foothills"] = {"Hinterlands", "Undercity", "Arathi Highlands"},
		["Moonglade"] = {"Felwood", "Winterspring"},
		["Orgrimmar"] = {"Ashenvale", "Azshara", "Dustwallow Marsh", "Felwood", "Thunder Bluff", "Barrens", "Winterspring", "Tanaris"},
		["Searing Gorge"] = {"Badlands", "Burning Steppes"},
		["Silithus"] = {"Tanaris"},
		["Silverpine Forest"] = {"Undercity"},
		["Stonetalon Mountains"] = {"Thunder Bluff", "Barrens", "Desolace"},
		["Stranglethorn Vale"] = {"Badlands", "Swamp of Sorrows"},
		["Swamp of Sorrows"] = {"Stranglethorn Vale", "Badlands"},
		["Tanaris"] = {"Barrens", "Orgrimmar", "Thunder Bluff", "Feralas", "Dustwallow Marsh", "Silithus"},
		["Barrens"] = {"Orgrimmar", "Thunder Bluff", "Azshara", "Thousand Needles", "Tanaris", "Stonetalon Mountains", "Ashenvale", "Dustwallow Marsh", "Feralas"},
		["Hinterlands"] = {"Undercity", "Hillsbrad Foothills"},
		["Undercity"] = {"Hinterlands", "Badlands", "Silverpine Forest", "Hillsbrad Foothills", "Eastern Plaguelands", "Arathi Highlands"},
		["Thousand Needles"] = {"Barrens", "Thunder Bluff"},
		["Thunder Bluff"] = {"Thousand Needles", "Barrens", "Tanaris", "Stonetalon Mountains", "Orgrimmar", "Feralas", "Desolace", "Dustwallow Marsh"},
		["Winterspring"] = {"Moonglade", "Orgrimmar", "Felwood"},
	},
};

------------------------------------------------------
-- load only if not already loaded
------------------------------------------------------

if (GFWZones == nil) then
	GFWZones = {};
end
local G = GFWZones;
if (G.Version == nil or (tonumber(G.Version) ~= nil and G.Version < GFWZONES_THIS_VERSION)) then

	-- load zone data
	if (G.AdjacentZones == nil) then
		G.AdjacentZones = {};
	end
	for aZone, adjacentZones in tempAdjacentZones do
		if (G.AdjacentZones[aZone] == nil) then
			G.AdjacentZones[aZone] = {};
		end
		G.AdjacentZones[aZone] = GFWTable.Merge(G.AdjacentZones[aZone], adjacentZones);
	end
	if (G.FlightZones == nil) then
		G.FlightZones = {};
	end
	for _, faction in {FACTION_ALLIANCE, FACTION_HORDE} do
		if (G.FlightZones[faction] == nil) then
			G.FlightZones[faction] = {};
		end
		for aZone, flightZones in tempFlightZones do
			if (G.FlightZones[faction][aZone] == nil) then
				G.FlightZones[faction][aZone] = {};
			end
			G.FlightZones[faction][aZone] = GFWTable.Merge(G.FlightZones[faction][aZone], flightZones);
		end
	end

	-- Functions
	G.NormalizeZoneName = GFWZones_temp_NormalizeZoneName;
	G.ConnectionsForZone = GFWZones_temp_ConnectionsForZone;
	
	-- Set version number
	G.Version = GFWZONES_THIS_VERSION;
end



