------------------------------------------------------
-- HuntersHelper.lua
-- Version 1800.2
------------------------------------------------------

-- Saved configuration & info
FHH_Config = { };
FHH_Config.Tooltip = true;

--FHH_AbilityInfo = { };
-- Has the following internal structure:
--		REALM_PLAYER = {
--			SKILL
--		}

-- Runtime state
FHH_State = { };
FHH_State.RealmPlayer = nil;
FHH_State.TamingCritter = nil;
FHH_State.TamingType = nil;

-- Constants
ALL_FAMILIES = "all beast families";
PET_TRAINER = "Pet Trainers (found in major cities and some towns)";
MAX_REPORTED_ZONES = 4;

function FHH_OnLoad()

	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("UPDATE_MOUSEOVER_UNIT");

	-- Register Slash Commands
	SLASH_FHH1 = "/huntershelper";
	SLASH_FHH2 = "/hh";
	SlashCmdList["FHH"] = function(msg)
		FHH_ChatCommandHandler(msg);
	end
	
	GFWUtils.Print("Fizzwidget Hunter's Helper initialized!");
	
end

function FHH_OnEvent(event)

	if ( event == "PLAYER_ENTERING_WORLD" ) then
		
		_, realClass = UnitClass("player");
		if (realClass == "HUNTER") then
			-- only do stuff related to taming and checking hunter spells if you're a hunter.
			this:RegisterEvent("UNIT_AURA");
			this:RegisterEvent("UNIT_NAME_UPDATE");
			this:RegisterEvent("CRAFT_SHOW");
			this:RegisterEvent("CHAT_MSG_SYSTEM");

			if (FHH_State.RealmPlayer == nil) then
				FHH_State.RealmPlayer = GetCVar("realmName") .. "." .. UnitName("player");
			end
			if (FHH_AbilityInfo == nil) then
				FHH_AbilityInfo = {};
			end
			if (FHH_AbilityInfo[FHH_State.RealmPlayer] == nil or table.getn(FHH_AbilityInfo[FHH_State.RealmPlayer]) == 0) then
				
				local _, myClass = UnitClass("player");
				if (myClass == "HUNTER" and UnitLevel("player") > 9) then
					FHH_AbilityInfo[FHH_State.RealmPlayer] = { };
					GFWUtils.Print("Hunter's Helper needs to collect info about what pet skills you already know; please open your Beast Training window. (Info on future skills will be collected as they are learned.)");
				end
			end
		end
		
	elseif ( event == "UPDATE_MOUSEOVER_UNIT" ) then
	
		if ( UnitExists("mouseover") and not UnitPlayerControlled("mouseover") and FHH_Config.Tooltip ) then

			local _, myClass = UnitClass("player");
			if (FHH_Config.Tooltip == "hunter" and myClass ~= "HUNTER") then return; end
			
			local creepName = UnitName("mouseover");
			local abilitiesList = nil;
			
			if (FHH_NewBeastInfo ~= nil and FHH_NewBeastInfo[creepName] ~= nil) then
				abilitiesList = GFWTable.Copy(FHH_NewBeastInfo[creepName]);
			elseif (FHH_BeastInfo[creepName] ~= nil) then
				abilitiesList = GFWTable.Copy(FHH_BeastInfo[creepName]);
				if (FHH_BadBeastInfo ~= nil and FHH_BadBeastInfo[creepName] ~= nil) then
					abilitiesList = GFWTable.Subtract(abilitiesList, FHH_BadBeastInfo[creepName]);
				end
			end
						
			if (abilitiesList) then
			
				local _, myClass = UnitClass("player");
				if (myClass == "HUNTER" and FHH_AbilityInfo ~= nil and FHH_AbilityInfo[FHH_State.RealmPlayer] ~= nil and table.getn(FHH_AbilityInfo[FHH_State.RealmPlayer]) > 0) then
					for index = 1, table.getn(abilitiesList) do
						if (GFWTable.IndexOf(FHH_AbilityInfo[FHH_State.RealmPlayer], abilitiesList[index]) ~= 0) then
							abilitiesList[index] = GRAY_FONT_COLOR_CODE..abilitiesList[index]..FONT_COLOR_CODE_CLOSE;
						else
							abilitiesList[index] = GREEN_FONT_COLOR_CODE..abilitiesList[index]..FONT_COLOR_CODE_CLOSE;
						end
					end
				end
						
				GameTooltip:AddLine("Abilities: "..table.concat(abilitiesList, ", "), 1.0, 1.0, 1.0);
				GameTooltip:SetHeight(GameTooltip:GetHeight() + 14);
				local width = 20 + getglobal(GameTooltip:GetName().."TextLeft"..GameTooltip:NumLines()):GetWidth();
				if ( GameTooltip:GetWidth() < width ) then
					GameTooltip:SetWidth(width);
				end
			end
			
			local creepLevel = UnitLevel("mouseover");
			local creepType = UnitClassification("mouseover");
			if (FHH_NewBeastLevels ~= nil and FHH_NewBeastLevels[creepName] ~= nil) then
				FHH_NewBeastLevels[creepName].min = math.min(FHH_NewBeastLevels[creepName].min, creepLevel);
				FHH_NewBeastLevels[creepName].max = math.max(FHH_NewBeastLevels[creepName].max, creepLevel);
				if (FHH_NewBeastLevels[creepName].type ~= nil and creepType ~= "normal") then
					FHH_NewBeastLevels[creepName].type = creepType;
				end
			elseif (FHH_BeastLevels[creepName] ~= nil) then
				if (creepLevel < FHH_BeastLevels[creepName].min or (FHH_BeastLevels[creepName].max ~= nil and creepLevel > FHH_BeastLevels[creepName].max)) then
					if (FHH_NewBeastLevels == nil) then
						FHH_NewBeastLevels = {};
					end
					FHH_NewBeastLevels[creepName] = {};
					FHH_NewBeastLevels[creepName].min = math.min(FHH_BeastLevels[creepName].min, creepLevel);
					FHH_NewBeastLevels[creepName].max = math.max(FHH_BeastLevels[creepName].max or FHH_BeastLevels[creepName].min, creepLevel);
				end
				if (creepType ~= "normal" and creepType ~= FHH_BeastLevels[creepName].type) then
					FHH_NewBeastLevels[creepName].type = creepType;
				end
			end

		end
		
	elseif ( event == "UNIT_AURA" ) then
	
		if ( arg1 == "player" and FHH_HasTameEffect("player") ) then
			FHH_State.TamingCritter = UnitName("target");
			FHH_State.TamingType = UnitClassification("target");
		end
		
	elseif ( event == "UNIT_NAME_UPDATE" ) then
	
		if ( arg1 == "pet" and FHH_State.TamingCritter ) then
			FHH_CheckPetSpells(FHH_State.TamingCritter);
			FHH_State.TamingCritter = nil;
			FHH_State.TamingType = nil;
		end
		
	elseif ( event == "CRAFT_SHOW" ) then

		-- Beast Training uses the CraftFrame; we can tell it's not really a craft because it doesn't have a skill-level bar.
		local name, rank, maxRank = GetCraftDisplaySkillLine();
		if ( name ) then return; end
		
		local numCrafts = GetNumCrafts();
		if not ( numCrafts > 0 )then return; end
		
		for i=1, numCrafts do
			local craftName, craftSubSpellName, _, _, _, _, requiredLevel = GetCraftInfo(i);
			_, _, rankNum = string.find(craftSubSpellName, "(%d)");
			local spellInfo = craftName.." "..rankNum;
			if (GFWTable.IndexOf(FHH_AbilityInfo[FHH_State.RealmPlayer], spellInfo) == 0) then
				table.insert(FHH_AbilityInfo[FHH_State.RealmPlayer], spellInfo)
			end;
			if ( requiredLevel and requiredLevel > 0 ) then
				local normalizedSpellName = string.lower(craftName);
				if (string.find(normalizedSpellName, "resist")) then
					normalizedSpellName = "resist"; -- all resistances have the same info; this lets us keep one table for them
				end
				FHH_RecordNewRequiredLevel(normalizedSpellName, tonumber(rankNum), requiredLevel, true);
			end
		end
		table.sort(FHH_AbilityInfo[FHH_State.RealmPlayer]);

	elseif ( event == "CHAT_MSG_SYSTEM" ) then

		_, _, spellName, rankNum = string.find(arg1, "You have learned a new spell: ([%w ]+) %(Rank (%d+)%)");
		if (spellName and rankNum and spellName ~= "" and rankNum ~= "" ) then
			normalizedSpellName = string.lower(spellName);
			if (string.find(normalizedSpellName, "resist")) then
				normalizedSpellName = "resist"; -- all resistances have the same info; this lets us keep one table for them
			end
			if (FHH_RequiredLevel[normalizedSpellName] ~= nil) then
				-- only track spells we know are hunter pet spells
				table.insert(FHH_AbilityInfo[FHH_State.RealmPlayer], spellName.." "..rankNum);
				table.sort(FHH_AbilityInfo[FHH_State.RealmPlayer]);
			end
		end
		
	end

end

function FHH_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print("Fizzwidget Hunter's Helper:");
		GFWUtils.Print("/huntershelper /hh <command>");
		GFWUtils.Print("- "..GFWUtils.Hilite("help").." - Print this helplist.");
		GFWUtils.Print("- "..GFWUtils.Hilite("on").." | "..GFWUtils.Hilite("off").." | "..GFWUtils.Hilite("onlyhunter").." - Turn display of beast abilities in tooltips on or off, or make them only appear if you're playing a hunter.");
		GFWUtils.Print("- "..GFWUtils.Hilite("status").." - Check current settings.");
		GFWUtils.Print("- "..GFWUtils.Hilite("find <ability> <rank>").." - List where beasts with a given ability (e.g. Bite 6) can be found.");
		return;
	end
	
	if (msg == "onlyhunter") then
		FHH_Config.Tooltip = "hunter";
		GFWUtils.Print("Hunter's Helper is enabled; beast tooltips will display ability info only when playing a Hunter character.");
		return;
	end
	if (msg == "on") then
		FHH_Config.Tooltip = true;
		GFWUtils.Print("Hunter's Helper is enabled; beast tooltips will display ability info.");
		return;
	end
	if (msg == "off") then
		FHH_Config.Tooltip = nil;
		GFWUtils.Print("Hunter's Helper is disabled; no extra info added to tooltips.");
		return;
	end

	-- Check Status
	if ( msg == "status" ) then
		if ( FHH_Config.Tooltip == "hunter" ) then
			GFWUtils.Print("Hunter's Helper is enabled; beast tooltips will display ability info only when playing as a Hunter.");
		elseif ( FHH_Config.Tooltip ) then
			GFWUtils.Print("Hunter's Helper is enabled; beast tooltips will display ability info.");
		else
			GFWUtils.Print("Hunter's Helper is disabled; no extra info added to tooltips.");
		end
		return;
	end
	
	if (msg == "reset") then
		FHH_Config = { };
		FHH_Config.Tooltip = true;
		FHH_AbilityInfo = nil;
		FHH_NewBeastInfo = nil;
		FHH_BadBeastInfo = nil;
		FHH_NewBeastLevels = nil;
		FHH_NewSpellInfo = nil;
		FHH_NewLearnableBy = nil;
		FHH_NewRequiredLevel = nil;	
		GFWUtils.Print("Hunter's Helper has been reset to default options and all stored data cleared.");
		return;
	end
		
	local _, _, cmd, spellName, rankNum = string.find(msg, "(find%w*) ([%l ]+) *(%d*)");
	if (cmd == "find" or cmd == "findall") then
		if (spellName == nil or spellName == "") then
			GFWUtils.Print("Usage: "..GFWUtils.Hilite("/hh find <ability> <rank>"));
			return;
		end
		
		spellName = string.lower(spellName);
		spellName = string.gsub(spellName, "^%s+", ""); -- strip leading spaces
		spellName = string.gsub(spellName, "%s+$", ""); -- and trailing spaces
		local capWords = function(first, rest) return string.upper(first)..string.lower(rest); end
		local niceSpellName = string.gsub(spellName, "(%l)([%w_']*)", capWords);
		if (string.find(spellName, "resist")) then
			spellName = "resist"; -- all resistances have the same info; this lets us keep one table for them
			niceSpellName = "Arcane/Fire/Frost/Nature/Shadow Resistance"
		end
		local spellInfo = FHH_SpellInfo[spellName];
		local newSpellInfo;
		if (FHH_NewSpellInfo ~= nil) then
			newSpellInfo = FHH_NewSpellInfo[spellName];
		end
		if (spellInfo == nil or (type(spellInfo) == "table" and GFWTable.Count(spellInfo) == 0)) then
			if (newSpellInfo == nil or GFWTable.Count(newSpellInfo) == 0) then
				GFWUtils.Print(GFWUtils.Hilite(niceSpellName).." is not a known beast ability.");
				return;
			else
				spellInfo = newSpellInfo;
			end
		end
		local rankTable = FHH_RequiredLevel[spellName];
		local newRankTable;
		if (FHH_NewRequiredLevel ~= nil) then
			newRankTable = FHH_NewRequiredLevel[spellName];
		end
		if (rankTable == nil or GFWTable.Count(rankTable) == 0) then
			if (newRankTable == nil or GFWTable.Count(newRankTable) == 0) then
				GFWUtils.Print(GFWUtils.Red("Hunter's Helper error:").." found "..GFWUtils.Hilite(niceSpellName).." but can't find rank info. Please report to gazmik@fizzwidget.com");
				return;
			else
				rankTable = newRankTable;
			end
		end
		
		rankNum = tonumber(rankNum);
		if (rankNum ~= nil) then
			if not (rankTable[rankNum]) then
				GFWUtils.Print(GFWUtils.Hilite(niceSpellName).." is not known to have a rank "..GFWUtils.Hilite(rankNum)..".");
				return;
			end
			
			-- report minimum pet level for ability
			local minLevel = rankTable[rankNum];
			local petLevel = 60;
			if (UnitExists("pet")) then
				petLevel = tonumber(UnitLevel("pet"));
			end
			if (minLevel == nil) then
				minLevel = newRankTable[rankNum];
			end
			if (minLevel == nil) then
				GFWUtils.Print(GFWUtils.Red("Hunter's Helper error:").." can't find required level for "..GFWUtils.Hilite(niceSpellName.." "..rankNum)..". Please report to gazmik@fizzwidget.com");
			else
				if (type(minLevel) == "string") then
					GFWUtils.Print(GFWUtils.Hilite(niceSpellName.." "..rankNum).." requires at least pet level "..GFWUtils.Hilite(minLevel)..". (Assumed because it was found on a beast of this level that you tamed. Open you Beast Training window and Hunter's Helper can collect more accurate information.)");
				elseif (petLevel >= minLevel) then
					GFWUtils.Print(GFWUtils.Hilite(niceSpellName.." "..rankNum).." requires pet level "..GFWUtils.Hilite(minLevel)..".");
				else
					GFWUtils.Print(GFWUtils.Hilite(niceSpellName.." "..rankNum).." requires pet level "..GFWUtils.Red(minLevel)..".");
				end
			end
		else
			local knownRanks = {};
			for rankNum in rankTable do
				table.insert(knownRanks, rankNum);
			end
			local newRanks = {};
			for rankNum in (newRankTable or {}) do
				table.insert(newRanks, rankNum);
			end
			local allRanks = GFWTable.Merge(knownRanks, newRanks);
			GFWUtils.Print("Ranks known about for "..GFWUtils.Hilite(niceSpellName)..": "..table.concat(allRanks, " "));
			if (type(spellInfo) ~= "string") then
				GFWUtils.Print("Type "..GFWUtils.Hilite("/hh find "..spellName).." and a number to get info about that rank.");
			end
		end
	
		-- report available creature families
		local families = FHH_LearnableBy[spellName];
		if (FHH_NewLearnableBy and FHH_NewLearnableBy[spellName]) then
			families = GFWTable.Merge(families, FHH_NewLearnableBy[spellName]);
			if (table.getn(GFWTable.Diff(families, FHH_AllFamilies)) == 0 ) then
				families = ALL_FAMILIES;
			end
		end
		if (families ~= nil or (type(families) == "table" and table.getn(families) == 0)) then
			if (type(families) == "string") then
				GFWUtils.Print(GFWUtils.Hilite(niceSpellName).." is learnable by "..GFWUtils.Hilite(families)..".");
			else
				local listText = table.concat(families, ", ");
				GFWUtils.Print(GFWUtils.Hilite(niceSpellName).." is learnable by: "..GFWUtils.Hilite(listText)..".");
			end
		end

		-- case 1: first levels of Growl are innate
		if (spellName == "growl" and rankNum ~= nil and rankNum <= 2) then
			GFWUtils.Print("You should already know "..GFWUtils.Hilite(niceSpellName.." "..rankNum).." if you've learned Beast Training.");
			return;
		end
		
		-- case 2: spells taught by trainers, for which rank doesn't matter
		if (type(spellInfo) == "string") then
			local spellSummary = niceSpellName;
			if (rankNum ~= nil) then
				spellSummary = spellSummary.." "..rankNum;
			end
			GFWUtils.Print(GFWUtils.Hilite(spellSummary).." is learned from "..spellInfo..".");
			return;
		end
		
		if (rankNum == nil) then return; end
		
		--case 3: lookup by spell and rank, report by zone (sanity check first)
		local spellRankInfo = spellInfo[rankNum];
		local maxZones = MAX_REPORTED_ZONES;
		if (cmd == "findall") then
			maxZones = 100; -- arbitrarily high so we find everything.
		end
		if (spellRankInfo == nil ) then
			GFWUtils.Print("Hunter's Helper doesn't know about any creatures with "..GFWUtils.Hilite(niceSpellName.." "..rankNum)..".");
			return;
		end
		GFWUtils.Print(GFWUtils.Hilite(niceSpellName.." "..rankNum).." can be learned from:");
		local numReportedZones = 0;
		local zoneName = GFWZones.NormalizeZoneName(GetRealZoneText());
		local critterList = {};
		if (spellRankInfo[zoneName] ~= nil and table.getn(spellRankInfo[zoneName]) > 0) then
			critterList = GFWTable.Merge(critterList, spellRankInfo[zoneName]);
		end
		if (FHH_NewSpellInfo and FHH_NewSpellInfo[spellName] and FHH_NewSpellInfo[spellName][rankNum] and FHH_NewSpellInfo[spellName][rankNum][zoneName]) then
			critterList = GFWTable.Merge(critterList, FHH_NewSpellInfo[spellName][rankNum][zoneName]);
		end
		if (table.getn(critterList) > 0) then
			GFWUtils.Print(zoneName..": "..GFWUtils.Hilite(FHH_CreatureListString(critterList)));
			numReportedZones = numReportedZones + 1;
		end
		
		local zoneConnections = GFWZones.ConnectionsForZone(zoneName);
		for _, zones in zoneConnections do
			for _, zoneName in zones do
				local critterList = {};
				if (spellRankInfo[zoneName] ~= nil and table.getn(spellRankInfo[zoneName]) > 0) then
					critterList = GFWTable.Merge(critterList, spellRankInfo[zoneName]);
				end
				if (FHH_NewSpellInfo and FHH_NewSpellInfo[spellName] and FHH_NewSpellInfo[spellName][rankNum] and FHH_NewSpellInfo[spellName][rankNum][zoneName]) then
					critterList = GFWTable.Merge(critterList, FHH_NewSpellInfo[spellName][rankNum][zoneName]);
				end
				if (table.getn(critterList) > 0) then
					GFWUtils.Print(zoneName..": "..GFWUtils.Hilite(FHH_CreatureListString(critterList)));
					numReportedZones = numReportedZones + 1;
					if (numReportedZones >= maxZones) then return; end
				end
			end
		end
		
		if (numReportedZones == 0) then
			-- if we get here, we think we know about the spell but can't find beasts with it in our table. this shouldn't happen.
			GFWUtils.Print(GFWUtils.Red("Hunter's Helper error:").." got spell info for "..GFWUtils.Hilite(niceSpellName.." "..rankNum).." but no zone info. Please report to gazmik@fizzwidget.com.");
		end
		return;
	end
	
	-- if we got all the way to here, we got invalid input.
	FHH_ChatCommandHandler("help");
	
end

function FHH_CreatureListString(critterList)
	local listString = ""
	for _, name in critterList do
		local info = FHH_BeastLevels[name];
		if (info == nil and FHH_NewBeastLevels ~= nil) then
			info = FHH_NewBeastLevels[name];
		end
		if (info == nil) then
			listString = listString..", ";
		else
			listString = listString .. name .. " ";
			local myLevel = UnitLevel("player");
			local minLevel = info.min;
			local maxLevel = info.max;
			if (info.min > UnitLevel("player")) then
				minLevel = RED_FONT_COLOR_CODE..info.min..FONT_COLOR_CODE_CLOSE;
			end
			if (info.max ~= nil and info.max > UnitLevel("player")) then
				maxLevel = RED_FONT_COLOR_CODE..info.max..FONT_COLOR_CODE_CLOSE;
			end
			if (info.min == info.max or info.max == nil) then			
				listString = listString.."("..minLevel;
			else
				listString = listString.."("..minLevel.."-"..maxLevel;
			end
			if (info.type == nil) then
				listString = listString.."), ";
			else
				listString = listString.." "..info.type.."), ";
			end				
		end
	end
	listString = string.gsub(listString, ", $", "");
	return listString;
end

function FHH_HasTameEffect(unit)

	local i = 1;
	local buff;
	buff = UnitBuff(unit, i);
	while buff do
		if ( string.find(buff, "Ability_Hunter_BeastTaming") ) then
			return true;
		end
		i = i + 1;
		buff = UnitBuff(unit, i);
	end
	return false;

end

function FHH_CheckPetSpells(critter)

	local CurrentPetSpells = { };
	local i = 1;
	local spellName, spellRank = GetSpellName(i, BOOKTYPE_PET);
	while spellName do
		_, _, rankNum = string.find(spellRank, "(%d)");
		table.insert(CurrentPetSpells, spellName.." "..rankNum);
		i = i + 1;
		spellName, spellRank = GetSpellName(i, BOOKTYPE_PET);
	end
	
	if ( table.getn(CurrentPetSpells) == 0 ) then return; end
	
	if ( FHH_BeastInfo[critter] ) then
	
		-- record any spells the critter has that our built-in table doesn't know about 
		local UnknownPetSpells = { };
		for _, spellName in CurrentPetSpells do
			if ( GFWTable.IndexOf(FHH_BeastInfo[critter], spellName) == 0 ) then
				table.insert(UnknownPetSpells, spellName);
			end
		end
		if ( table.getn(UnknownPetSpells) > 0 ) then
			if (FHH_NewBeastInfo == nil) then
				FHH_NewBeastInfo = {};
			end
			FHH_NewBeastInfo[critter] = CurrentPetSpells; -- we want to remember the entire current spells list
		end
		
		-- record any spells our built-in table thinks the critter has, but the critter actually doesn't
		local WrongPetSpells = { };
		for _, spellName in FHH_BeastInfo[critter] do
			if ( GFWTable.IndexOf(CurrentPetSpells, spellName) == 0 ) then
				table.insert(WrongPetSpells, spellName);
			end
		end
		if ( table.getn(WrongPetSpells) > 0 ) then
			if (FHH_BadBeastInfo == nil) then
				FHH_BadBeastInfo = {};
			end
			FHH_BadBeastInfo[critter] = WrongPetSpells;
		end
		
		if ((FHH_NewBeastInfo and FHH_NewBeastInfo[critter]) or (FHH_BadBeastInfo and FHH_BadBeastInfo[critter])) then
			local details = "(expected "..table.concat(FHH_BeastInfo[critter], ", ").."; found "..table.concat(CurrentPetSpells, ", ")..").";
			GFWUtils.Print("Hunter's Helper has incorrect data on "..GFWUtils.Hilite(critter.." "..details).." Please submit a correction to gazmik@fizzwidget.com.)");
		end
		
	else
	
		-- this pet is entirely new to our list
		if (FHH_NewBeastInfo == nil) then
			FHH_NewBeastInfo = {};
		end
		FHH_NewBeastInfo[critter] = CurrentPetSpells;
		FHH_RecordNewBeastLevel(critter, UnitLevel("pet"), FHH_State.TamingType);
		
		local details = " (found "..table.concat(CurrentPetSpells, ", ").." in "..GetRealZoneText()..").";
		GFWUtils.Print("Hunter's Helper has no data on "..GFWUtils.Hilite(critter.." "..details).." Please submit a correction to gazmik@fizzwidget.com.)");

	end
	
	for _, niceSpellName in CurrentPetSpells do
		local _, _, spellKey, rankNum = string.find(niceSpellName, "^([%w ]+) (%d+)$");
		if (spellKey == nil or spellKey == "" or tonumber(rankNum) == nil) then
			GFWUtils.Print(GFWUtils.Red("Hunter's Helper Error: ").."Can't parse spell "..GFWUtils.Hilite(niceSpellName).." from "..GFWUtils.Hilite(critter)..".");
		else
			spellKey = string.lower(spellKey);
			rankNum = tonumber(rankNum);
			local level = UnitLevel("pet");
			local family = UnitCreatureFamily("pet");

			FHH_RecordNewSpellInfo(spellKey, rankNum, critter);
			FHH_RecordNewRequiredFamily(spellKey, family);
			FHH_RecordNewRequiredLevel(spellKey, rankNum, level);
		end
	end

end

function FHH_RecordNewSpellInfo(spellKey, rankNum, critter)
	if (FHH_SpellInfo[spellKey] and FHH_SpellInfo[spellKey][rankNum]) then
		for zoneName, beastsTable in FHH_SpellInfo[spellKey][rankNum] do
			for _, aBeast in beastsTable do
				if (aBeast == critter) then
					return; -- we've already recorded this in our static data
				end
			end
		end
	end
	
	if (FHH_NewSpellInfo == nil) then
		FHH_NewSpellInfo = {};
	end
	if (FHH_NewSpellInfo[spellKey] == nil) then
		FHH_NewSpellInfo[spellKey] = {};
	end
	if (FHH_NewSpellInfo[spellKey][rankNum] == nil) then
		FHH_NewSpellInfo[spellKey][rankNum] = {};
	end
	local currentZone = GFWZones.NormalizeZoneName(GetRealZoneText());
	if (FHH_NewSpellInfo[spellKey][rankNum][currentZone] == nil) then
		FHH_NewSpellInfo[spellKey][rankNum][currentZone] = {};
	end
	if (not GFWTable.KeyOf(FHH_NewSpellInfo[spellKey][rankNum][currentZone], critter)) then
		table.insert(FHH_NewSpellInfo[spellKey][rankNum][currentZone], critter);
	end
end

function FHH_RecordNewRequiredFamily(spellKey, family)
	if (FHH_LearnableBy[spellKey] and GFWTable.KeyOf(FHH_LearnableBy[spellKey], family)) then
		return; -- we've already recorded this in our static data
	end
	
	if (FHH_NewLearnableBy == nil) then
		FHH_NewLearnableBy = {};
	end
	if (FHH_NewLearnableBy[spellKey] == nil) then
		FHH_NewLearnableBy[spellKey] = {};
	end
	if (not GFWTable.KeyOf(FHH_NewLearnableBy[spellKey], family)) then
		table.insert(FHH_NewLearnableBy[spellKey], family);
	end
end

function FHH_RecordNewRequiredLevel(spellKey, rankNum, level, verified)	
	if (FHH_RequiredLevel[spellKey] and FHH_RequiredLevel[spellKey][rankNum]) then
		return; -- we've already recorded this in our static data
	end
	
	if (FHH_NewRequiredLevel == nil) then
		FHH_NewRequiredLevel = {};
	end
	if (FHH_NewRequiredLevel[spellKey] == nil) then
		FHH_NewRequiredLevel[spellKey] = {};
	end
	if (verified) then
		FHH_NewRequiredLevel[spellKey][rankNum] = level;
	elseif (FHH_NewRequiredLevel[spellKey][rankNum] == nil) then
		FHH_NewRequiredLevel[spellKey][rankNum] = tostring(level);
	else
		local existingRank = FHH_NewRequiredLevel[spellKey][rankNum];
		if (type(existingRank) == "string") then
			-- we don't have a certain answer yet, we'll use what we just got to refine it
			FHH_NewRequiredLevel[spellKey][rankNum] = tostring(math.min(level, tonumber(existingRank)));
		end
	end
end

function FHH_RecordNewBeastLevel(creepName, level, type)
	local minLevel, maxLevel, mobType;
	if (FHH_NewBeastLevels == nil) then
		FHH_NewBeastLevels = {};
	end
	if (FHH_NewBeastLevels[creepName] == nil) then
		FHH_NewBeastLevels[creepName] = {};
	else
		minLevel = FHH_NewBeastLevels[creepName].min;
		maxLevel = FHH_NewBeastLevels[creepName].max;
		mobType = FHH_NewBeastLevels[creepName].type;
	end
	FHH_NewBeastLevels[creepName].min = math.min(minLevel or level, level);
	FHH_NewBeastLevels[creepName].max = math.max(maxLevel or level, level);
	if (mobType ~= "normal" and type ~= "normal") then
		FHH_NewBeastLevels[creepName].type = type;
	end
end